<?php

/**
 *  * You are allowed to use this API in your web application.
 *
 * Copyright (C) 2018 by customweb GmbH
 *
 * This program is licenced under the customweb software licence. With the
 * purchase or the installation of the software in your application you
 * accept the licence agreement. The allowed usage is outlined in the
 * customweb software licence which can be found under
 * http://www.sellxed.com/en/software-license-agreement
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at http://www.sellxed.com/shop.
 *
 * See the customweb software licence agreement for more details.
 *
 */

//require_once 'Customweb/Stripe/Method/CreditCard/Validator.php';
//require_once 'Customweb/Stripe/Authorization/Transaction.php';
//require_once 'Customweb/Form/Element.php';
//require_once 'Customweb/I18n/Translation.php';
//require_once 'Customweb/Stripe/Method/Default.php';
//require_once 'Customweb/Form/Control/Html.php';
//require_once 'Customweb/Stripe/Method/CreditCard/AliasValidator.php';
//require_once 'Customweb/Stripe/Util.php';


/**
 *
 * @author Sebastian Bossert
 * @Method(paymentMethods={'creditcard', 'visa', 'mastercard', 'americanexpress', 'discovercard', 'diners', 'jcb'})
 */
class Customweb_Stripe_Method_CreditCard_Method extends Customweb_Stripe_Method_Default {

	public function getVisibleFormFields(Customweb_Payment_Authorization_IOrderContext $orderContext, $aliasTransaction, $failedTransaction, $paymentCustomerContext){
		$fields = array();
		$fields[] = $this->getJavascriptField();
		if ($aliasTransaction instanceof Customweb_Stripe_Authorization_Transaction) {
			$control = new Customweb_Form_Control_Html('stripe-alias-display', $aliasTransaction->getAliasForDisplay());
			$control->addValidator(new Customweb_Stripe_Method_CreditCard_AliasValidator($control, $this->getPaymentMethodName()));
			$element = new Customweb_Form_Element(Customweb_I18n_Translation::__('Selected card:'), $control);
			$element->appendJavaScript($this->getAliasSourceJavascript($aliasTransaction));
			$fields[] = $element;
		}
		else {
			$fields[] = $this->getCardInputTemplateField($orderContext);
		}
		return $fields;
	}

	public function extractAliasForDisplay(array $json){
		return 'XXXX XXXX XXXX ' . $json['card']['last4'];
	}

	protected function getSupportedBrands(){
		if ($this->existsPaymentMethodConfigurationValue('credit_card_brands')) {
			return $this->getPaymentMethodConfigurationValue('credit_card_brands');
		}
		else {
			$parameters = $this->getPaymentMethodParameters();
			if (!isset($parameters['brand'])) {
				throw new Exception("Brand could not be determined.");
			}
			return array(
				$parameters['brand'] 
			);
		}
	}

	protected function getJavascriptPrecondition(Customweb_Stripe_Authorization_Transaction $transaction){
		$context = $transaction->getTransactionContext();
		/* @var $context Customweb_Payment_Authorization_Ajax_ITransactionContext */
		$failedFunction = $context->getJavaScriptFailedCallbackFunction();
		$successFunction = $context->getJavaScriptSuccessCallbackFunction();
		
		$prefix = 'stripe' . $this->getPaymentMethodName();
		
		$zeroCheckout = Customweb_Stripe_Util::isZeroTransaction($transaction) ? 'true' : 'false';
		
		$preconditions = '';
		$authorizeUrl = $this->getContainer()->getSecuredEndpoint($transaction, 'ajax-authorize');
		$failedUrl = $this->getContainer()->getSecuredEndpoint($transaction, 'ajax-failure');
		
		$useOptionalThreeD = $this->useOptionalThreeD() ? 'true' : 'false';
		$preconditions = "if(!document.{$prefix}Source) {
		($failedFunction)('$failedUrl&message=source+not+set+in+javascript');
		return;
	}
	if ($zeroCheckout || (document.{$prefix}Source.card.three_d_secure == 'optional' && !$useOptionalThreeD) || document.{$prefix}Source.card.three_d_secure == 'not_supported' ) {
		($successFunction)('$authorizeUrl&source=' + encodeURIComponent(document.{$prefix}Source.id));
		return;
	}";
		
		return $preconditions;
	}

	public function useOptionalThreeD(){
		return $this->existsPaymentMethodConfigurationValue('optional_3d') && $this->getPaymentMethodConfigurationValue('optional_3d') == 'force';
	}

	protected function getSourceCreationJson(Customweb_Stripe_Authorization_Transaction $transaction){
		$json = json_encode($this->getSourceCreationData($transaction));
		// card source may either be a javascript variable. json_encode always encapsulates the string in double quotations, which means we must manually replace it.
		return str_replace('"CW_PLACEHOLDER_CARD"', $this->getCardSource($transaction), $json);
	}

	protected function getSourceCreationData(Customweb_Stripe_Authorization_Transaction $transaction){
		$data = parent::getSourceCreationData($transaction);
		$data['type'] = 'three_d_secure';
		$data['three_d_secure'] = array(
			'card' => 'CW_PLACEHOLDER_CARD' 
		);
		return $data;
	}

	private function getCardInputTemplateField(Customweb_Payment_Authorization_IOrderContext $orderContext){
		$control = new Customweb_Form_Control_Html('stripe-card',
				"<div class='StripeWrapper'><div id='stripe{$this->getPaymentMethodName()}-card-element' class='StripeElement'></div></div>");
		$control->setRequired(false);
		$control->addValidator(
				new Customweb_Stripe_Method_CreditCard_Validator($control, $orderContext->getBillingAddress(),
						$orderContext->getOrderAmountInDecimals(), $orderContext->getCurrencyCode(), $this->getPaymentMethodName(),
						$orderContext->getCheckoutId(), $this->getContainer()->getConfiguration()->getShopId()));
		$field = new Customweb_Form_Element(Customweb_I18n_Translation::__("Card"), $control);
		$field->setRequired(false);
		
		$field->appendJavaScript($this->getMountElementsJavascript($orderContext));
		return $field;
	}

	private function getMountElementsJavascript(Customweb_Payment_Authorization_IOrderContext $orderContext){
		$supported_brands = json_encode(array_values($this->getSupportedBrands()));
		$prefix = 'stripe' . $this->getPaymentMethodName();
		$locale = $orderContext->getLanguage()->getIetfCode();
		
		return "function {$prefix}InitializeCCField() {
	if(typeof document.{$prefix}Stripe === 'undefined') {
		setTimeout({$prefix}InitializeCCField, 500);
	}
	else {
		document.{$prefix}Card = document.{$prefix}Stripe.elements({locale: \"{$locale}\"}).create('card', {hidePostalCode: true});
		document.{$prefix}Card.mount('#{$prefix}-card-element');
		document.{$prefix}Card.on('change', function(event) {
			document.{$prefix}SupportedBrand = ($supported_brands.indexOf(event.brand) !== -1);
		});
	}
}
{$prefix}InitializeCCField();";
	}

	private function getCardSource(Customweb_Stripe_Authorization_Transaction $transaction){
		$alias = $transaction->getTransactionContext()->getAlias();
		
		if ($alias instanceof Customweb_Stripe_Authorization_Transaction) {
			$cardSource = $alias->getAlias();
			return "'$cardSource'";
		}
		$prefix = 'stripe' . $this->getPaymentMethodName();
		
		return "document.{$prefix}Source.id";
	}
}