<?php

/**
 *  * You are allowed to use this API in your web application.
 *
 * Copyright (C) 2018 by customweb GmbH
 *
 * This program is licenced under the customweb software licence. With the
 * purchase or the installation of the software in your application you
 * accept the licence agreement. The allowed usage is outlined in the
 * customweb software licence which can be found under
 * http://www.sellxed.com/en/software-license-agreement
 *
 * Any modification or distribution is strictly forbidden. The license
 * grants you the installation in one application. For multiuse you will need
 * to purchase further licences at http://www.sellxed.com/shop.
 *
 * See the customweb software licence agreement for more details.
 *
 */

//require_once 'Customweb/Form/Validator/IValidator.php';
//require_once 'Customweb/I18n/Translation.php';
//require_once 'Customweb/Stripe/Util.php';


/**
 */
class Customweb_Stripe_Method_CreditCard_Validator implements Customweb_Form_Validator_IValidator {
	private $control;
	private $paymentMethodName;
	private $address;
	private $amount;
	private $currency;
	private $checkoutId;
	private $shopId;

	public function __construct(Customweb_Form_Control_IControl $control, Customweb_Payment_Authorization_OrderContext_IAddress $address, $amount, $currency, $paymentMethodName, $checkoutId, $shopId){
		$this->control = $control;
		$this->paymentMethodName = $paymentMethodName;
		$this->address = $address;
		$this->currency = $currency;
		$this->amount = $amount;
		$this->checkoutId = $checkoutId;
		$this->shopId = $shopId;
	}

	/**
	 * The control object on which the validation is executed.
	 *
	 * @return Customweb_Form_Control_IControl
	 */
	public function getControl(){
		return $this->control;
	}

	/**
	 * This method must return the JavaScript code required to
	 * execute the validation.
	 * The code must return a anonymous
	 * JS function which accepts two arguments:
	 *
	 * <ol>
	 * <li>resultCallback: A callback function which accepts two arguments:
	 * The first argument is the result of the validation
	 * The second argument is an error message if the validation failed
	 *
	 * <li>element: The HTML element (control field) on which the validation
	 * should be exectued on.</li>
	 * </ol>
	 */
	public function getCallbackJs(){
		$options = array(
			'owner' => array(
				'name' => $this->address->getFirstName() . ' ' . $this->address->getLastName(),
				'address' => array(
					'line1' => $this->address->getStreet(),
					'city' => $this->address->getCity(),
					'postal_code' => $this->address->getPostCode(),
					'country' => $this->address->getCountryIsoCode() 
				) 
			),
			'metadata' => array(
				'cw_transaction_id' => 'not_set-' . $this->checkoutId,
				'cw_shop_id' => $this->shopId 
			) 
		);
		
		$tmp = trim($this->address->getState());
		if (!empty($tmp)) {
			$options['owner']['address']['state'] = $tmp;
		}
		$tmp = trim($this->address->getMobilePhoneNumber());
		if (!empty($tmp)) {
			$options['owner']['phone'] = $tmp;
		}
		$tmp = trim($this->address->getPhoneNumber());
		if (!empty($tmp)) {
			$options['owner']['phone'] = $tmp;
		}
		
		$options = Customweb_Stripe_Util::simpleJSEncode($options);
		$prefix = 'stripe' . $this->paymentMethodName;
		$brandNotSupported = Customweb_I18n_Translation::__("The credit card is not of a supported brand.");
		$js = "function (resultCallback, element) {
	if(!document.{$prefix}SupportedBrand) {
		resultCallback(false, \"$brandNotSupported\");
	}
	else {	
		var options = $options;
		document.{$prefix}Stripe.createSource(document.{$prefix}Card, options).then(function(result) {
			if(result.error) {
				resultCallback(false, result.error.message);
			}
			else {
				document.{$prefix}Source = result.source;
				resultCallback(true);
			}
		});
	}
}";
		return $js;
	}
}